/* 
 * 1Gb API
 *
 *  API для работы с хостингом позволяет автоматизировать управление ресурсами и анализ данных.  ## Примеры использования API хостинга:  * Отслеживание состояния аккаунта и своевременный анализ уведомлений * Автоматическое создание, удаление, конфигурирование ресурсов * Доступ к данным внутреннего мониторинга 1Gb.ru, анализ качества работы VDS * Автоматизация типовых реакций на проблемы ресурсов и хостинга   Доступ к API хостинга активен в любом оплаченном тарифном плане. Для тестовых или бесплатных аккаунтов API не работает.   ## Принципы работы и примеры  Доступ происходит по протоколу HTTPS по базовому адресу https://www.1gb.ru/api/. Параметры большинства запросов можно передавать с помощью GET или POST параметров. Ответ содержит объект или массив объектов по стандарту JSON.  Перед началом работы с API требуется получить token для дальнейшей авторизации. Это делается следующим образом (приведены примеры использования GET параметров):  1. https://www.1gb.ru/api/auth/start?login=[логин_кабинета] - возвращает в ответ string, содержащий соль 2. https://www.1gb.ru/api/auth/login?login=[логин_кабинета]&salt=[соль]&response=[response] - возвращает в ответ string, содержащий token; псевдокод для вычисления response на PHP: *md5($password.$salt.\"\\n\")* (сумма md5 строки \"пароль+соль+символ-перевода-строки\")  Последний запрос возвращает string, в нем token, его нужно запомнить и использовать для последующей авторизации. Token утрачивает силу при смене главного пароля аккаунта, существует также возможность деактивировать отдельные token-ы со страницы кабинета \"смена главного пароля аккаунта\".  Примеры дальнейших запросов:      https://www.1gb.ru/api/billing/info?_token_=[запомненный token] - получение общих данных аккаунта   Ответы API с HTTP кодом 200 обозначают успешное исполнение операции. В ответном JSON может содержаться два разных типа:  * [ \"string\" ]: либо просто \"OK\", либо \"[число]\" (необходимо привести к int), если возвращается ID созданного объекта * [ { ... } , { ... } ] (массив объектов): если запрос возвращает список из 0 или более структур (хеш \"поле\":\"значение\").   Другие коды (больше 200) обозначают ошибку, при этом тело ответа содержит string с объяснением, пример: \"ERROR: vds.missing\".  ## Команды API и общие соглашения  Большинство команд API, таких как /list, возвращают внутренние структуры из базы данных хостинга, которые содержат сотни полей. Документировать их трудоемко, поэтому рекомендуем просто смотреть на вывод соответствующих команд и анализировать структуры данных по очевидным названиям полей.  В том случае, если трактовка полей вызывает затруднения, пишите на support@1gb.ru, поможем разобраться.  ## Общедоступные на данный момент команды  https://www.1gb.ru/api + адрес из списка ниже. Обязательный параметр каждой команды (кроме этапа авторизации) - _token_. Остальные параметры приведены после команды.  
 *
 * OpenAPI spec version: 2.0.0
 * 
 * Generated by: https://github.com/swagger-api/swagger-codegen.git
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package swagger

import (
	"bytes"
	"fmt"
	"path/filepath"
	"reflect"
	"strings"
	"net/url"
	"github.com/go-resty/resty"
)

type APIClient struct {
}

func (c *APIClient) SelectHeaderContentType(contentTypes []string) string {

	if len(contentTypes) == 0 {
		return ""
	}
	if contains(contentTypes, "application/json") {
		return "application/json"
	}
	return contentTypes[0] // use the first content type specified in 'consumes'
}

func (c *APIClient) SelectHeaderAccept(accepts []string) string {

	if len(accepts) == 0 {
		return ""
	}
	if contains(accepts, "application/json") {
		return "application/json"
	}
	return strings.Join(accepts, ",")
}

func contains(source []string, containvalue string) bool {
	for _, a := range source {
		if strings.ToLower(a) == strings.ToLower(containvalue) {
			return true
		}
	}
	return false
}

func (c *APIClient) CallAPI(path string, method string,
	postBody interface{},
	headerParams map[string]string,
	queryParams url.Values,
	formParams map[string]string,
	fileName string,
	fileBytes []byte) (*resty.Response, error) {

	//set debug flag
	configuration := NewConfiguration()
	resty.SetDebug(configuration.GetDebug())

	request := prepareRequest(postBody, headerParams, queryParams, formParams, fileName, fileBytes)

	switch strings.ToUpper(method) {
	case "GET":
		response, err := request.Get(path)
		return response, err
	case "POST":
		response, err := request.Post(path)
		return response, err
	case "PUT":
		response, err := request.Put(path)
		return response, err
	case "PATCH":
		response, err := request.Patch(path)
		return response, err
	case "DELETE":
		response, err := request.Delete(path)
		return response, err
	}

	return nil, fmt.Errorf("invalid method %v", method)
}

func (c *APIClient) ParameterToString(obj interface{},collectionFormat string) string {
	if reflect.TypeOf(obj).String() == "[]string" {
		switch	collectionFormat {
		case "pipes":
			return strings.Join(obj.([]string), "|")
		case "ssv":
			return strings.Join(obj.([]string), " ")
		case "tsv":
			return strings.Join(obj.([]string), "\t")	
		case "csv" :
			return strings.Join(obj.([]string), ",")
		}
	}

	return obj.(string)
}

func prepareRequest(postBody interface{},
	headerParams map[string]string,
	queryParams url.Values,
	formParams map[string]string,
	fileName string,
	fileBytes []byte) *resty.Request {

	request := resty.R()
	request.SetBody(postBody)

	// add header parameter, if any
	if len(headerParams) > 0 {
		request.SetHeaders(headerParams)
	}

	// add query parameter, if any
	if len(queryParams) > 0 {
		request.SetMultiValueQueryParams(queryParams)
	}

	// add form parameter, if any
	if len(formParams) > 0 {
		request.SetFormData(formParams)
	}

	if len(fileBytes) > 0 && fileName != "" {
		_, fileNm := filepath.Split(fileName)
		request.SetFileReader("file", fileNm, bytes.NewReader(fileBytes))
	}
	return request
}
